@@ -8,13 +8,14 @@ use Carp qw/croak/;
 use JSON::MaybeXS;
 use LWP::UserAgent;
 
-use OpenAI::API::Request::Chat;
-use OpenAI::API::Request::Completion;
-use OpenAI::API::Request::Edit;
-use OpenAI::API::Request::Embedding;
-use OpenAI::API::Request::Moderation;
+use OpenAI::API::Resource::Chat;
+use OpenAI::API::Resource::Completion;
+use OpenAI::API::Resource::Edit;
+use OpenAI::API::Resource::Embedding;
+use OpenAI::API::Resource::Model;
+use OpenAI::API::Resource::Moderation;
 
-our $VERSION = 0.22;
+our $VERSION = 0.23;
 
 my $DEFAULT_API_BASE = 'https://api.openai.com/v1';
 my $DEFAULT_TIMEOUT  = 60;
@@ -39,39 +40,62 @@ sub new {
 
 sub chat {
     my ( $self, %params ) = @_;
-    my $request = OpenAI::API::Request::Chat->new( \%params );
+    my $request = OpenAI::API::Resource::Chat->new( \%params );
     return $self->_post( $request );
 }
 
 sub completions {
     my ( $self, %params ) = @_;
-    my $request = OpenAI::API::Request::Completion->new( \%params );
+    my $request = OpenAI::API::Resource::Completion->new( \%params );
     return $self->_post( $request );
 }
 
 sub edits {
     my ( $self, %params ) = @_;
-    my $request = OpenAI::API::Request::Edit->new( \%params );
+    my $request = OpenAI::API::Resource::Edit->new( \%params );
     return $self->_post( $request );
 }
 
 sub embeddings {
     my ( $self, %params ) = @_;
-    my $request = OpenAI::API::Request::Embedding->new( \%params );
+    my $request = OpenAI::API::Resource::Embedding->new( \%params );
     return $self->_post( $request );
 }
 
 sub moderations {
     my ( $self, %params ) = @_;
-    my $request = OpenAI::API::Request::Moderation->new( \%params );
+    my $request = OpenAI::API::Resource::Moderation->new( \%params );
     return $self->_post( $request );
 }
 
-sub _post {
-    my ( $self, $request ) = @_;
+sub models {
+    my ( $self, %params ) = @_;
+    my $request = OpenAI::API::Resource::Model->new( \%params );
+    return $self->_get($request);
+}
 
-    my $method = $request->endpoint();
-    my %params = %{$request};
+sub _get {
+    my ( $self, $resource ) = @_;
+
+    my $method = $resource->endpoint();
+    my %params = %{$resource};
+
+    my $req = HTTP::Request->new(
+        GET => "$self->{api_base}/$method",
+        [
+            'Content-Type'  => 'application/json',
+            'Authorization' => "Bearer $self->{api_key}",
+        ],
+    );
+
+    return $self->_http_send_request($req);
+}
+
+sub _post {
+    my ( $self, $resource ) = @_;
+
+    my $method = $resource->endpoint();
+    my %params = %{$resource};
 
     my $req = HTTP::Request->new(
         POST => "$self->{api_base}/$method",
@@ -82,6 +106,12 @@ sub _post {
         encode_json(\%params),
     );
 
+    return $self->_http_send_request($req);
+}
+
+sub _http_send_request {
+    my ($self, $req) = @_;
+
     for my $attempt ( 1 .. $self->{retry} ) {
         my $res = $self->{ua}->request($req);
 
@@ -90,7 +120,7 @@ sub _post {
         } elsif ( $res->code =~ /^(?:500|503|504|599)$/ && $attempt < $self->{retry} ) {
             sleep( $self->{sleep} );
         } else {
-            die "Error retrieving '$method': " . $res->status_line;
+            die "Error: '@{[ $res->status_line ]}'";
         }
     }
 }
@@ -217,7 +247,7 @@ Mandatory parameters:
 
 =back
 
-More info: L<OpenAI::API::Request::Chat>
+More info: L<OpenAI::API::Resource::Chat>
 
 =head2 completions()
 
@@ -233,7 +263,7 @@ Mandatory parameters:
 
 =back
 
-More info: L<OpenAI::API::Request::Completion>
+More info: L<OpenAI::API::Resource::Completion>
 
 =head2 edits()
 
@@ -251,7 +281,7 @@ Mandatory parameters:
 
 =back
 
-More info: L<OpenAI::API::Request::Edit>
+More info: L<OpenAI::API::Resource::Edit>
 
 =head2 embeddings()
 
@@ -268,7 +298,7 @@ Mandatory parameters:
 
 =back
 
-More info: L<OpenAI::API::Request::Embedding>
+More info: L<OpenAI::API::Resource::Embedding>
 
 =head2 moderations()
 
@@ -283,7 +313,7 @@ Mandatory parameters:
 
 =back
 
-More info: L<OpenAI::API::Request::Moderation>
+More info: L<OpenAI::API::Resource::Moderation>
 
 =head1 SEE ALSO
 
